/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_OMP_COMMUNICATION_TEST_H
#define CUBELIB_BSPOP_HYBRID_OMP_COMMUNICATION_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridMPICommunicationEfficiencyTest.h"
#include "BSPOPHybridCommunicationEfficiencyTest.h"


namespace bscanalysis
{
class BSPOPHybridOMPCommunicationEfficiencyTest : public popcalculation::PerformanceTest
{
private:

    BSPOPHybridCommunicationEfficiencyTest*    par_comm_eff;
    BSPOPHybridMPICommunicationEfficiencyTest* mpi_comm_eff;

    void
    calculate();

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );



    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "OpenMP Communication Efficiency can be evaluated directly by following formula:\n"
            "\tOpenMP Communication Efficiency = max(computation time)/max(time outside of MPI)\n"
            "\t                                = Hybrid Communication Efficiency / MPI Communication Efficiency";
        std::string not_active_text =
            "OpenMP Communication Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    BSPOPHybridOMPCommunicationEfficiencyTest( cube::CubeProxy*,
                                               BSPOPHybridCommunicationEfficiencyTest*,
                                               BSPOPHybridMPICommunicationEfficiencyTest* );

    virtual
    ~BSPOPHybridOMPCommunicationEfficiencyTest()
    {
    };

// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( par_comm_eff != nullptr && mpi_comm_eff != nullptr )
        {
            prereqs.push_back( par_comm_eff );
            prereqs.push_back( mpi_comm_eff );
        }
        return prereqs;
    }
};
};
#endif // BSPOPHybrid_IMBALANCE_TEST_H
