/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_OMP_LOAD_BALANCE_EFFICIENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_OMP_LOAD_BALANCE_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridMPILoadBalanceTest.h"
#include "BSPOPHybridLoadBalanceTest.h"


namespace bscanalysis
{
class BSPOPHybridOMPLoadBalanceEfficiencyTest : public popcalculation::PerformanceTest
{
private:

    BSPOPHybridLoadBalanceTest*    par_lb_eff;
    BSPOPHybridMPILoadBalanceTest* mpi_lb_eff;

    void
    calculate();

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );



    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "OpenMP Load Balance Efficiency can be computed as follows: \n"
            "OpenMP Load Balance Efficiency = Hybrid Load Balance Efficiency / MPI Load Balance Efficiency ";
        std::string not_active_text =
            "OpenMP Load Balance Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridOMPLoadBalanceEfficiencyTest(     cube::CubeProxy*,
                                                 BSPOPHybridLoadBalanceTest*,
                                                 BSPOPHybridMPILoadBalanceTest*
                                                 );

    virtual
    ~BSPOPHybridOMPLoadBalanceEfficiencyTest()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( par_lb_eff != nullptr && mpi_lb_eff != nullptr )
        {
            prereqs.push_back( par_lb_eff );
            prereqs.push_back( mpi_lb_eff );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
