/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_COMPUTATION_TIME_H
#define CUBELIB_POP_COMPUTATION_TIME_H

#include <string>
#include "PerformanceTest.h"


namespace popcalculation
{
class POPComputationTime : public PerformanceTest
{
private:
    cube::Metric* comp;

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Computation time shows the time spent in the part of the code, identified as a useful computation.\n"
            "Computation time is defined in form of exclusion, namely, computation time is part of the execution time spend NOT in MPI, \n"
            "NOT in OpenMP, NOT in IO, not in SHMEM, not in service libraries, insturmented using the library wrapping, NOT in CUDA, NOT OpenCL, \n"
            "NOT in OpenACC, NOT in OpenCL and so on. \n";
        std::string not_active_text =
            "Computation time metric is available for every Score-P/Scalasca measurement, where Time metric is present. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time and hence Computation Time .\n"
            "In this case this metric reported as NAN.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPComputationTime( cube::CubeProxy* );

    virtual
    ~POPComputationTime()
    {
    };
// ------ overview tests ---------


    virtual
    inline
    bool
    isPercent() const
    {
        return false;
    }

    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    inline
    bool
    isNormalized() const // returns true, if range goes from 0 to 1. false -> otherwise
    {
        return false;
    }

    inline
    virtual
    std::string
    units() const
    {
        return "sec";
    }
};
};
#endif // POP_ipc_TEST_H
